'use strict';

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _web3_tools = require('../utils/web3_tools');

var _TimeMock = require('../../build/contracts/TimeMock.json');

var _TimeMock2 = _interopRequireDefault(_TimeMock);

var _deploy = require('../../test/helpers/deploy');

var _deploy2 = _interopRequireDefault(_deploy);

var _consts = require('../consts');

var _bn = require('bn.js');

var _bn2 = _interopRequireDefault(_bn);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var web3 = void 0; /*
                   Copyright: Ambrosus Technologies GmbH
                   Email: tech@ambrosus.com
                   
                   This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
                   
                   This Source Code Form is “Incompatible With Secondary Licenses”, as defined by the Mozilla Public License, v. 2.0.
                   */

var gasUsed = void 0;
var from = void 0;
var hermes = void 0;
var perseus = void 0;
var atlasInitialShelterer = void 0;
var atlasQuickResolver = void 0;
var atlasSlowResolver = void 0;
var challenges = void 0;
var time = void 0;
var kycWhitelist = void 0;
var roles = void 0;
var fees = void 0;
var uploads = void 0;

var now = 1500000000;
var storagePeriods = 1;
var bundleId = '0xbeef';

var runGasBenchmark = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
    var feeForUpload, systemChallengeId, feeForChallenge, userChallengeId;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return setupEnvironment();

          case 2:

            console.log('\n Basic AMB-NET operations gas costs:\n');

            _context.t0 = getGasUsed;
            _context.next = 6;
            return onboardAsHermes(hermes);

          case 6:
            _context.t1 = _context.sent;
            gasUsed = (0, _context.t0)(_context.t1);

            console.log('                  Hermes onboarding: ' + gasUsed);

            _context.t2 = getGasUsed;
            _context.next = 12;
            return onboardAsAtlas(atlasInitialShelterer);

          case 12:
            _context.t3 = _context.sent;
            gasUsed = (0, _context.t2)(_context.t3);

            console.log('                   Atlas onboarding: ' + gasUsed);

            _context.next = 17;
            return onboardAsAtlas(atlasQuickResolver);

          case 17:
            _context.next = 19;
            return onboardAsAtlas(atlasSlowResolver);

          case 19:
            _context.next = 21;
            return getFeeForUpload(storagePeriods);

          case 21:
            feeForUpload = _context.sent;
            _context.t4 = getGasUsed;
            _context.next = 25;
            return registerBundle(hermes, bundleId, storagePeriods, feeForUpload);

          case 25:
            _context.t5 = _context.sent;
            gasUsed = (0, _context.t4)(_context.t5);

            console.log('                    Register bundle: ' + gasUsed);

            _context.next = 30;
            return lastChallengeId();

          case 30:
            systemChallengeId = _context.sent;
            _context.t6 = getGasUsed;
            _context.next = 34;
            return resolveChallenge(systemChallengeId, atlasInitialShelterer);

          case 34:
            _context.t7 = _context.sent;
            gasUsed = (0, _context.t6)(_context.t7);

            console.log('           Resolve system challenge: ' + gasUsed);

            _context.next = 39;
            return getFeeForChallenge(storagePeriods);

          case 39:
            feeForChallenge = _context.sent;
            _context.t8 = getGasUsed;
            _context.next = 43;
            return startChallenge(atlasInitialShelterer, bundleId, perseus, feeForChallenge);

          case 43:
            _context.t9 = _context.sent;
            gasUsed = (0, _context.t8)(_context.t9);

            console.log('               Start user challenge: ' + gasUsed);

            _context.next = 48;
            return lastChallengeId();

          case 48:
            userChallengeId = _context.sent;
            _context.t10 = getGasUsed;
            _context.next = 52;
            return resolveChallenge(userChallengeId, atlasQuickResolver);

          case 52:
            _context.t11 = _context.sent;
            gasUsed = (0, _context.t10)(_context.t11);

            console.log('Successfully resolve user challenge: ' + gasUsed);

            _context.prev = 55;
            _context.next = 58;
            return resolveChallenge(systemChallengeId, atlasInitialShelterer);

          case 58:
            _context.next = 64;
            break;

          case 60:
            _context.prev = 60;
            _context.t12 = _context['catch'](55);

            gasUsed = extractGasUsedFromError(_context.t12);
            console.log('      Fail resolving user challenge: ' + gasUsed);

          case 64:

            console.log('\n');

          case 65:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined, [[55, 60]]);
  }));

  return function runGasBenchmark() {
    return _ref.apply(this, arguments);
  };
}();

var setupEnvironment = function () {
  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
    var _ref3, _ref4, _ref5;

    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return (0, _web3_tools.createWeb3)();

          case 2:
            web3 = _context2.sent;
            _context2.next = 5;
            return web3.eth.getAccounts();

          case 5:
            _ref3 = _context2.sent;
            _ref4 = (0, _slicedToArray3.default)(_ref3, 6);
            from = _ref4[0];
            hermes = _ref4[1];
            perseus = _ref4[2];
            atlasInitialShelterer = _ref4[3];
            atlasQuickResolver = _ref4[4];
            atlasSlowResolver = _ref4[5];
            _context2.next = 15;
            return (0, _deploy2.default)({
              web3: web3,
              contracts: {
                challenges: true,
                bundleStore: true,
                fees: true,
                sheltering: true,
                uploads: true,
                atlasStakeStore: true,
                apolloDepositStore: true,
                time: _TimeMock2.default,
                roles: true,
                kycWhitelist: true,
                config: true,
                payouts: true,
                payoutsStore: true,
                rolesStore: true,
                shelteringTransfersStore: true,
                challengesStore: true
              } });

          case 15:
            _ref5 = _context2.sent;
            challenges = _ref5.challenges;
            time = _ref5.time;
            kycWhitelist = _ref5.kycWhitelist;
            roles = _ref5.roles;
            fees = _ref5.fees;
            uploads = _ref5.uploads;
            _context2.next = 24;
            return setTimestamp(now);

          case 24:
            _context2.next = 26;
            return kycWhitelist.methods.add(atlasInitialShelterer, _consts.ATLAS, _consts.ATLAS1_STAKE).send({ from: from });

          case 26:
            _context2.next = 28;
            return kycWhitelist.methods.add(atlasQuickResolver, _consts.ATLAS, _consts.ATLAS1_STAKE).send({ from: from });

          case 28:
            _context2.next = 30;
            return kycWhitelist.methods.add(atlasSlowResolver, _consts.ATLAS, _consts.ATLAS1_STAKE).send({ from: from });

          case 30:
            _context2.next = 32;
            return kycWhitelist.methods.add(hermes, _consts.HERMES, 0).send({ from: from });

          case 32:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  }));

  return function setupEnvironment() {
    return _ref2.apply(this, arguments);
  };
}();

var setTimestamp = function () {
  var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(timestamp) {
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            return _context3.abrupt('return', time.methods.setCurrentTimestamp(timestamp).send({ from: from }));

          case 1:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  }));

  return function setTimestamp(_x) {
    return _ref6.apply(this, arguments);
  };
}();

var onboardAsAtlas = function () {
  var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(address) {
    var url;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            url = address + '@atlasUrl.com';
            _context4.next = 3;
            return roles.methods.onboardAsAtlas(url).send({ from: address, value: _consts.ATLAS1_STAKE });

          case 3:
            return _context4.abrupt('return', _context4.sent);

          case 4:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  }));

  return function onboardAsAtlas(_x2) {
    return _ref7.apply(this, arguments);
  };
}();

var onboardAsHermes = function () {
  var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(address) {
    var url;
    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            url = address + '@hermesUrl.com';
            _context5.next = 3;
            return roles.methods.onboardAsHermes(url).send({ from: address });

          case 3:
            return _context5.abrupt('return', _context5.sent);

          case 4:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, undefined);
  }));

  return function onboardAsHermes(_x3) {
    return _ref8.apply(this, arguments);
  };
}();

var getFeeForUpload = function () {
  var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(storagePeriods) {
    return _regenerator2.default.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.t0 = _bn2.default;
            _context6.next = 3;
            return fees.methods.getFeeForUpload(storagePeriods).call();

          case 3:
            _context6.t1 = _context6.sent;
            return _context6.abrupt('return', new _context6.t0(_context6.t1));

          case 5:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, undefined);
  }));

  return function getFeeForUpload(_x4) {
    return _ref9.apply(this, arguments);
  };
}();

var registerBundle = function () {
  var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(uploaderId, bundleId, storagePeriods, fee) {
    return _regenerator2.default.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            return _context7.abrupt('return', uploads.methods.registerBundle(bundleId, storagePeriods).send({ from: uploaderId, value: fee }));

          case 1:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, undefined);
  }));

  return function registerBundle(_x5, _x6, _x7, _x8) {
    return _ref10.apply(this, arguments);
  };
}();

var resolveChallenge = function () {
  var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(challengeId, resolverId) {
    return _regenerator2.default.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            return _context8.abrupt('return', challenges.methods.resolve(challengeId).send({ from: resolverId }));

          case 1:
          case 'end':
            return _context8.stop();
        }
      }
    }, _callee8, undefined);
  }));

  return function resolveChallenge(_x9, _x10) {
    return _ref11.apply(this, arguments);
  };
}();

var getFeeForChallenge = function () {
  var _ref12 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9(storagePeriods) {
    return _regenerator2.default.wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            _context9.t0 = _bn2.default;
            _context9.next = 3;
            return fees.methods.getFeeForChallenge(storagePeriods).call();

          case 3:
            _context9.t1 = _context9.sent;
            return _context9.abrupt('return', new _context9.t0(_context9.t1));

          case 5:
          case 'end':
            return _context9.stop();
        }
      }
    }, _callee9, undefined);
  }));

  return function getFeeForChallenge(_x11) {
    return _ref12.apply(this, arguments);
  };
}();

var startChallenge = function () {
  var _ref13 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10(sheltererId, bundleId, challengerId, fee) {
    return _regenerator2.default.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            return _context10.abrupt('return', challenges.methods.start(sheltererId, bundleId).send({ from: challengerId, value: fee }));

          case 1:
          case 'end':
            return _context10.stop();
        }
      }
    }, _callee10, undefined);
  }));

  return function startChallenge(_x12, _x13, _x14, _x15) {
    return _ref13.apply(this, arguments);
  };
}();

var lastChallengeId = function () {
  var _ref14 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11() {
    var _ref15, _ref16, challengeCreationEvent;

    return _regenerator2.default.wrap(function _callee11$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            _context11.next = 2;
            return challenges.getPastEvents('allEvents');

          case 2:
            _ref15 = _context11.sent;
            _ref16 = (0, _slicedToArray3.default)(_ref15, 1);
            challengeCreationEvent = _ref16[0];
            return _context11.abrupt('return', challengeCreationEvent.returnValues.challengeId);

          case 6:
          case 'end':
            return _context11.stop();
        }
      }
    }, _callee11, undefined);
  }));

  return function lastChallengeId() {
    return _ref14.apply(this, arguments);
  };
}();

var getGasUsed = function getGasUsed(tx) {
  return tx.gasUsed;
};

var extractGasUsedFromError = function extractGasUsedFromError(error) {
  var message = error.message;

  var regex = /"gasUsed": (\d+)/g;
  var result = regex.exec(message);
  return result ? result[1] : undefined;
};

runGasBenchmark();